@@ -47,24 +47,28 @@ class core_component {
     /** @var array list plugin types that support subplugins, do not add more here unless absolutely necessary */
     protected static $supportsubplugins = array('mod', 'editor', 'tool', 'local');
 
-    /** @var null cache of plugin types */
+    /** @var array cache of plugin types */
     protected static $plugintypes = null;
-    /** @var null cache of plugin locations */
+    /** @var array cache of plugin locations */
     protected static $plugins = null;
-    /** @var null cache of core subsystems */
+    /** @var array cache of core subsystems */
     protected static $subsystems = null;
-    /** @var null subplugin type parents */
+    /** @var array subplugin type parents */
     protected static $parents = null;
-    /** @var null subplugins */
+    /** @var array subplugins */
     protected static $subplugins = null;
-    /** @var null list of all known classes that can be autoloaded */
+    /** @var array list of all known classes that can be autoloaded */
     protected static $classmap = null;
-    /** @var null list of some known files that can be included. */
+    /** @var array list of all classes that have been renamed to be autoloaded */
+    protected static $classmaprenames = null;
+    /** @var array list of some known files that can be included. */
     protected static $filemap = null;
     /** @var int|float core version. */
     protected static $version = null;
     /** @var array list of the files to map. */
     protected static $filestomap = array('lib.php', 'settings.php');
+    /** @var array cache of PSR loadable systems */
+    protected static $psrclassmap = null;
 
     /**
      * Class loader for Frankenstyle named classes in standard locations.
@@ -92,6 +96,21 @@ class core_component {
             include_once(self::$classmap[$classname]);
             return;
         }
+        if (isset(self::$classmaprenames[$classname]) && isset(self::$classmap[self::$classmaprenames[$classname]])) {
+            $newclassname = self::$classmaprenames[$classname];
+            $debugging = "Class '%s' has been renamed for the autoloader and is now deprecated. Please use '%s' instead.";
+            debugging(sprintf($debugging, $classname, $newclassname), DEBUG_DEVELOPER);
+            class_alias($newclassname, $classname);
+            return;
+        }
+
+        // Attempt to normalize the classname.
+        $normalizedclassname = str_replace(array('/', '\\'), '_', $classname);
+        if (isset(self::$psrclassmap[$normalizedclassname])) {
+            // Function include would be faster, but for BC it is better to include only once.
+            include_once(self::$psrclassmap[$normalizedclassname]);
+            return;
+        }
     }
 
     /**
@@ -125,13 +144,15 @@ class core_component {
                 }
                 $cache = array();
                 include($cachefile);
-                self::$plugintypes = $cache['plugintypes'];
-                self::$plugins     = $cache['plugins'];
-                self::$subsystems  = $cache['subsystems'];
-                self::$parents     = $cache['parents'];
-                self::$subplugins  = $cache['subplugins'];
-                self::$classmap    = $cache['classmap'];
-                self::$filemap     = $cache['filemap'];
+                self::$plugintypes      = $cache['plugintypes'];
+                self::$plugins          = $cache['plugins'];
+                self::$subsystems       = $cache['subsystems'];
+                self::$parents          = $cache['parents'];
+                self::$subplugins       = $cache['subplugins'];
+                self::$classmap         = $cache['classmap'];
+                self::$classmaprenames  = $cache['classmaprenames'];
+                self::$filemap          = $cache['filemap'];
+                self::$psrclassmap      = $cache['psrclassmap'];
                 return;
             }
 
@@ -164,13 +185,15 @@ class core_component {
                     // $CFG->dirroot was changed.
                 } else {
                     // The cache looks ok, let's use it.
-                    self::$plugintypes = $cache['plugintypes'];
-                    self::$plugins     = $cache['plugins'];
-                    self::$subsystems  = $cache['subsystems'];
-                    self::$parents     = $cache['parents'];
-                    self::$subplugins  = $cache['subplugins'];
-                    self::$classmap    = $cache['classmap'];
-                    self::$filemap     = $cache['filemap'];
+                    self::$plugintypes      = $cache['plugintypes'];
+                    self::$plugins          = $cache['plugins'];
+                    self::$subsystems       = $cache['subsystems'];
+                    self::$parents          = $cache['parents'];
+                    self::$subplugins       = $cache['subplugins'];
+                    self::$classmap         = $cache['classmap'];
+                    self::$classmaprenames  = $cache['classmaprenames'];
+                    self::$filemap          = $cache['filemap'];
+                    self::$psrclassmap      = $cache['psrclassmap'];
                     return;
                 }
                 // Note: we do not verify $CFG->admin here intentionally,
@@ -249,14 +272,16 @@ class core_component {
         }
 
         $cache = array(
-            'subsystems'  => self::$subsystems,
-            'plugintypes' => self::$plugintypes,
-            'plugins'     => self::$plugins,
-            'parents'     => self::$parents,
-            'subplugins'  => self::$subplugins,
-            'classmap'    => self::$classmap,
-            'filemap'     => self::$filemap,
-            'version'     => self::$version,
+            'subsystems'        => self::$subsystems,
+            'plugintypes'       => self::$plugintypes,
+            'plugins'           => self::$plugins,
+            'parents'           => self::$parents,
+            'subplugins'        => self::$subplugins,
+            'classmap'          => self::$classmap,
+            'classmaprenames'   => self::$classmaprenames,
+            'filemap'           => self::$filemap,
+            'version'           => self::$version,
+            'psrclassmap'       => self::$psrclassmap,
         );
 
         return '<?php
@@ -278,7 +303,9 @@ $cache = '.var_export($cache, true).';
         }
 
         self::fill_classmap_cache();
+        self::fill_classmap_renames_cache();
         self::fill_filemap_cache();
+        self::fill_psr_cache();
         self::fetch_core_version();
     }
 
@@ -578,14 +605,9 @@ $cache = '.var_export($cache, true).';
                 self::load_classes($plugintype.'_'.$pluginname, "$fulldir/classes");
             }
         }
-
-        // Note: Add extra deprecated legacy classes here as necessary.
-        self::$classmap['textlib'] = "$CFG->dirroot/lib/classes/text.php";
-        self::$classmap['collatorlib'] = "$CFG->dirroot/lib/classes/collator.php";
         ksort(self::$classmap);
     }
 
-
     /**
      * Fills up the cache defining what plugins have certain files.
      *
@@ -654,6 +676,78 @@ $cache = '.var_export($cache, true).';
         unset($items);
     }
 
+    /**
+     * Fill caches for classes following the PSR-0 standard for the
+     * specified Vendors.
+     *
+     * PSR Autoloading is detailed at http://www.php-fig.org/psr/psr-0/.
+     */
+    protected static function fill_psr_cache() {
+        global $CFG;
+
+        $psrsystems = array(
+            'Horde' => 'horde/framework',
+        );
+        self::$psrclassmap = array();
+
+        foreach ($psrsystems as $system => $fulldir) {
+            if (!$fulldir) {
+                continue;
+            }
+            self::load_psr_classes($CFG->libdir . DIRECTORY_SEPARATOR . $fulldir);
+        }
+    }
+
+    /**
+     * Find all PSR-0 style classes in within the base directory.
+     *
+     * @param string $basedir The base directory that the PSR-type library can be found in.
+     * @param string $subdir The directory within the basedir to search for classes within.
+     */
+    protected static function load_psr_classes($basedir, $subdir = null) {
+        if ($subdir) {
+            $fulldir = implode(DIRECTORY_SEPARATOR, array($basedir, $subdir));
+            $classnameprefix = preg_replace('/\//', '_', $subdir);
+        } else {
+            $fulldir = $basedir;
+        }
+        if (!is_dir($fulldir)) {
+            return;
+        }
+
+        $items = new \DirectoryIterator($fulldir);
+        foreach ($items as $item) {
+            if ($item->isDot()) {
+                continue;
+            }
+            if ($item->isDir()) {
+                $dirname = $item->getFilename();
+                $newsubdir = $dirname;
+                if ($subdir) {
+                    $newsubdir = implode(DIRECTORY_SEPARATOR, array($subdir, $dirname));
+                }
+                self::load_psr_classes($basedir, $newsubdir);
+                continue;
+            }
+
+            $filename = $item->getFilename();
+            $classname = preg_replace('/\.php$/', '', $filename);
+
+            if ($filename === $classname) {
+                // Not a php file.
+                continue;
+            }
+
+            if ($classnameprefix) {
+                $classname = $classnameprefix . '_' . $classname;
+            }
+
+            self::$psrclassmap[$classname] = $fulldir . DIRECTORY_SEPARATOR . $filename;
+        }
+        unset($item);
+        unset($items);
+    }
+
     /**
      * List all core subsystems and their location
      *
@@ -854,6 +948,22 @@ $cache = '.var_export($cache, true).';
         }
     }
 
+    /**
+     * Normalize the component name.
+     *
+     * Note: this does not verify the validity of the plugin or component.
+     *
+     * @param string $component
+     * @return string
+     */
+    public static function normalize_componentname($componentname) {
+        list($plugintype, $pluginname) = self::normalize_component($componentname);
+        if ($plugintype === 'core' && is_null($pluginname)) {
+            return $plugintype;
+        }
+        return $plugintype . '_' . $pluginname;
+    }
+
     /**
      * Normalize the component name using the "frankenstyle" rules.
      *
@@ -1025,4 +1135,47 @@ $cache = '.var_export($cache, true).';
     public static function is_core_subsystem($subsystemname) {
         return isset(self::$subsystems[$subsystemname]);
     }
+
+    /**
+     * Records all class renames that have been made to facilitate autoloading.
+     */
+    protected static function fill_classmap_renames_cache() {
+        global $CFG;
+
+        self::$classmaprenames = array();
+
+        self::load_renamed_classes("$CFG->dirroot/lib/");
+
+        foreach (self::$subsystems as $subsystem => $fulldir) {
+            self::load_renamed_classes($fulldir);
+        }
+
+        foreach (self::$plugins as $plugintype => $plugins) {
+            foreach ($plugins as $pluginname => $fulldir) {
+                self::load_renamed_classes($fulldir);
+            }
+        }
+    }
+
+    /**
+     * Loads the db/renamedclasses.php file from the given directory.
+     *
+     * The renamedclasses.php should contain a key => value array ($renamedclasses) where the key is old class name,
+     * and the value is the new class name.
+     * It is only included when we are populating the component cache. After that is not needed.
+     *
+     * @param string $fulldir
+     */
+    protected static function load_renamed_classes($fulldir) {
+        $file = $fulldir . '/db/renamedclasses.php';
+        if (is_readable($file)) {
+            $renamedclasses = null;
+            require($file);
+            if (is_array($renamedclasses)) {
+                foreach ($renamedclasses as $oldclass => $newclass) {
+                    self::$classmaprenames[(string)$oldclass] = (string)$newclass;
+                }
+            }
+        }
+    }
 }
