@@ -462,5 +462,124 @@ function xmldb_assign_upgrade($oldversion) {
     // Moodle v2.6.0 release upgrade line.
     // Put any upgrade step following this.
 
+    if ($oldversion < 2014010801) {
+
+        // Define field sendstudentnotifications to be added to assign.
+        $table = new xmldb_table('assign');
+        $field = new xmldb_field('sendstudentnotifications',
+                                 XMLDB_TYPE_INTEGER,
+                                 '2',
+                                 null,
+                                 XMLDB_NOTNULL,
+                                 null,
+                                 '1',
+                                 'markingallocation');
+
+        // Conditionally launch add field sendstudentnotifications.
+        if (!$dbman->field_exists($table, $field)) {
+            $dbman->add_field($table, $field);
+        }
+
+        // Assign savepoint reached.
+        upgrade_mod_savepoint(true, 2014010801, 'assign');
+    }
+
+    // Moodle v2.7.0 release upgrade line.
+    // Put any upgrade step following this.
+
+    if ($oldversion < 2014051201) {
+
+        // Cleanup bad database records where assignid is missing.
+
+        $DB->delete_records('assign_user_mapping', array('assignment'=>0));
+        // Assign savepoint reached.
+        upgrade_mod_savepoint(true, 2014051201, 'assign');
+    }
+    if ($oldversion < 2014072400) {
+
+        // Add "latest" column to submissions table to mark the latest attempt.
+        $table = new xmldb_table('assign_submission');
+        $field = new xmldb_field('latest', XMLDB_TYPE_INTEGER, '2', null, XMLDB_NOTNULL, null, '0', 'attemptnumber');
+
+        // Conditionally launch add field latest.
+        if (!$dbman->field_exists($table, $field)) {
+            $dbman->add_field($table, $field);
+        }
+
+        // Assign savepoint reached.
+        upgrade_mod_savepoint(true, 2014072400, 'assign');
+    }
+    if ($oldversion < 2014072401) {
+
+         // Define index latestattempt (not unique) to be added to assign_submission.
+        $table = new xmldb_table('assign_submission');
+        $index = new xmldb_index('latestattempt', XMLDB_INDEX_NOTUNIQUE, array('assignment', 'userid', 'groupid', 'latest'));
+
+        // Conditionally launch add index latestattempt.
+        if (!$dbman->index_exists($table, $index)) {
+            $dbman->add_index($table, $index);
+        }
+
+        // Assign savepoint reached.
+        upgrade_mod_savepoint(true, 2014072401, 'assign');
+    }
+    if ($oldversion < 2014072405) {
+
+        // Prevent running this multiple times.
+
+        $countsql = 'SELECT COUNT(id) FROM {assign_submission} WHERE latest = ?';
+
+        $count = $DB->count_records_sql($countsql, array(1));
+        if ($count == 0) {
+
+            // Mark the latest attempt for every submission in mod_assign.
+            $maxattemptsql = 'SELECT assignment, userid, groupid, max(attemptnumber) AS maxattempt
+                                FROM {assign_submission}
+                            GROUP BY assignment, groupid, userid';
+
+            // Note: souterouter looks redundant below, but it forces
+            // MySQL to use an in memory table to store the results of the
+            // inner query. Without this MySQL would complain that the UPDATE
+            // is operating on the same table as the FROM (which is true).
+            $maxattemptidssql = 'SELECT souterouter.id FROM (
+                                    SELECT souter.id
+                                       FROM {assign_submission} souter
+                                       JOIN (' . $maxattemptsql . ') sinner
+                                         ON souter.assignment = sinner.assignment
+                                        AND souter.userid = sinner.userid
+                                        AND souter.groupid = sinner.groupid
+                                        AND souter.attemptnumber = sinner.maxattempt
+                                ) souterouter';
+            $select = 'id IN(' . $maxattemptidssql . ')';
+            $DB->set_field_select('assign_submission', 'latest', 1, $select);
+
+            // Look for grade records with no submission record.
+            // This is when a teacher has marked a student before they submitted anything.
+            $records = $DB->get_records_sql('SELECT g.id, g.assignment, g.userid
+                                               FROM {assign_grades} g
+                                          LEFT JOIN {assign_submission} s
+                                                 ON s.assignment = g.assignment
+                                                AND s.userid = g.userid
+                                              WHERE s.id IS NULL');
+            $submissions = array();
+            foreach ($records as $record) {
+                $submission = new stdClass();
+                $submission->assignment = $record->assignment;
+                $submission->userid = $record->userid;
+                $submission->status = 'new';
+                $submission->groupid = 0;
+                $submission->latest = 1;
+                $submission->timecreated = time();
+                $submission->timemodified = time();
+                array_push($submissions, $submission);
+            }
+
+            $DB->insert_records('assign_submission', $submissions);
+        }
+
+        // Assign savepoint reached.
+        upgrade_mod_savepoint(true, 2014072405, 'assign');
+    }
+
     return true;
 }
