@@ -179,10 +179,9 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         // Setup the assignment.
         $this->create_extra_users();
         $this->setUser($this->editingteachers[0]);
-        $time = time();
         $assign = $this->create_instance(array(
             'assignsubmission_onlinetext_enabled'=>1,
-            'duedate' => $time - 4 * 24 * 60 * 60,
+            'duedate' => time() - 4 * 24 * 60 * 60,
          ));
         $PAGE->set_url(new moodle_url('/mod/assign/view.php', array(
             'id' => $assign->get_course_module()->id,
@@ -193,17 +192,15 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         $gradingtable = new assign_grading_table($assign, 1, '', 0, true);
         $output = $assign->get_renderer()->render($gradingtable);
         $this->assertContains(get_string('submissionstatus_', 'assign'), $output);
-        $difftime = time() - $time;
-        $this->assertContains(get_string('overdue', 'assign', format_time(4*24*60*60 + $difftime)), $output);
+        $this->assertContains(get_string('overdue', 'assign', format_time(4*24*60*60)), $output);
 
         // Grant an extension that is in the past.
-        $assign->testable_save_user_extension($this->students[0]->id, $time - 2 * 24 * 60 * 60);
+        $assign->testable_save_user_extension($this->students[0]->id, time() - 2 * 24 * 60 * 60);
         $gradingtable = new assign_grading_table($assign, 1, '', 0, true);
         $output = $assign->get_renderer()->render($gradingtable);
         $this->assertContains(get_string('submissionstatus_', 'assign'), $output);
-        $this->assertContains(get_string('userextensiondate', 'assign', userdate($time - 2*24*60*60)), $output);
-        $difftime = time() - $time;
-        $this->assertContains(get_string('overdue', 'assign', format_time(2*24*60*60 + $difftime)), $output);
+        $this->assertContains(get_string('userextensiondate', 'assign', userdate(time() - 2*24*60*60)), $output);
+        $this->assertContains(get_string('overdue', 'assign', format_time(2*24*60*60)), $output);
 
         // Simulate a submission.
         $this->setUser($this->students[0]);
@@ -216,17 +213,14 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
                                          'format'=>FORMAT_MOODLE);
         $plugin = $assign->get_submission_plugin_by_type('onlinetext');
         $plugin->save($submission, $data);
-        $submittedtime = time();
 
         // Verify output.
         $this->setUser($this->editingteachers[0]);
         $gradingtable = new assign_grading_table($assign, 1, '', 0, true);
         $output = $assign->get_renderer()->render($gradingtable);
         $this->assertContains(get_string('submissionstatus_submitted', 'assign'), $output);
-        $this->assertContains(get_string('userextensiondate', 'assign', userdate($time - 2*24*60*60)), $output);
-
-        $difftime = $submittedtime - $time;
-        $this->assertContains(get_string('submittedlateshort', 'assign', format_time(2*24*60*60 + $difftime)), $output);
+        $this->assertContains(get_string('userextensiondate', 'assign', userdate(time() - 2*24*60*60)), $output);
+        $this->assertContains(get_string('submittedlateshort', 'assign', format_time(2*24*60*60)), $output);
     }
 
     /**
@@ -590,6 +584,8 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
     }
 
     public function test_list_participants() {
+        global $CFG, $DB;
+
         $this->create_extra_users();
         $this->setUser($this->editingteachers[0]);
         $assign = $this->create_instance(array('grade'=>100));
@@ -607,28 +603,30 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         set_user_preference('grade_report_showonlyactiveenrol', false);
         $assign = $this->create_instance(array('grade'=>100));
         $this->assertEquals(self::DEFAULT_STUDENT_COUNT + self::EXTRA_STUDENT_COUNT, count($assign->list_participants(null, true)));
+
+        // Turn on availability and a group restriction, and check that it doesn't
+        // show users who aren't in the group.
+        $CFG->enableavailability = true;
+        $specialgroup = $this->getDataGenerator()->create_group(
+                array('courseid' => $this->course->id));
+        $assign = $this->create_instance(array('grade' => 100,
+                'availability' => json_encode(\core_availability\tree::get_root_json(
+                    array(\availability_group\condition::get_json($specialgroup->id))))));
+        groups_add_member($specialgroup, $this->students[0]);
+        groups_add_member($specialgroup, $this->students[1]);
+        $this->assertEquals(2, count($assign->list_participants(null, true)));
     }
 
     public function test_count_teams() {
         $this->create_extra_users();
         $this->setUser($this->editingteachers[0]);
-        $assign1 = $this->create_instance(array('teamsubmission' => 1));
-        $this->assertEquals(self::GROUP_COUNT + 1, $assign1->count_teams());
+        $assign = $this->create_instance(array('teamsubmission'=>1));
 
-        $grouping = $this->getDataGenerator()->create_grouping(array('courseid' => $this->course->id));
-        $this->getDataGenerator()->create_grouping_group(array('groupid' => $this->groups[0]->id, 'groupingid' => $grouping->id));
-        $this->getDataGenerator()->create_grouping_group(array('groupid' => $this->groups[1]->id, 'groupingid' => $grouping->id));
-
-        // No active group => 2 groups + the default one.
-        $assign2 = $this->create_instance(array('teamsubmission' => 1, 'teamsubmissiongroupingid' => $grouping->id));
-        $this->assertEquals(3, $assign2->count_teams());
-
-        // An active group => Just the selected one.
-        $this->assertEquals(1, $assign2->count_teams($this->groups[0]->id));
+        $this->assertEquals(self::GROUP_COUNT + 1, $assign->count_teams());
     }
 
     public function test_submit_to_default_group() {
-        global $DB, $SESSION;
+        global $DB;
 
         $this->preventResetByRollback();
         $sink = $this->redirectMessages();
@@ -636,8 +634,7 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         $this->setUser($this->editingteachers[0]);
         $params = array('teamsubmission' => 1,
                         'assignsubmission_onlinetext_enabled' => 1,
-                        'submissiondrafts' => 0,
-                        'groupmode' => VISIBLEGROUPS);
+                        'submissiondrafts'=>0);
         $assign = $this->create_instance($params);
 
         $newstudent = $this->getDataGenerator()->create_user();
@@ -657,142 +654,93 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         $assign->save_submission($data, $notices);
         $this->assertEmpty($notices, 'No errors on save submission');
 
-        // Set active groups to all groups.
-        $this->setUser($this->teachers[0]);
-        $SESSION->activegroup[$this->course->id]['aag'][0] = 0;
-        $this->assertEquals(1, $assign->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_SUBMITTED));
-
-        // Set an active group.
-        $anothergroup = $this->groups[0];
-        $SESSION->activegroup[$this->course->id]['aag'][0] = (int)$anothergroup->id;
-        $this->assertEquals(0, $assign->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_SUBMITTED));
-
         $sink->close();
     }
 
     public function test_count_submissions() {
-        global $SESSION;
-
         $this->create_extra_users();
         $this->setUser($this->editingteachers[0]);
-        $assign1 = $this->create_instance(array('assignsubmission_onlinetext_enabled' => 1));
+        $assign = $this->create_instance(array('assignsubmission_onlinetext_enabled'=>1));
 
         // Simulate a submission.
         $this->setUser($this->extrastudents[0]);
-        $submission = $assign1->get_user_submission($this->extrastudents[0]->id, true);
+        $submission = $assign->get_user_submission($this->extrastudents[0]->id, true);
         $submission->status = ASSIGN_SUBMISSION_STATUS_DRAFT;
-        $assign1->testable_update_submission($submission, $this->extrastudents[0]->id, true, false);
+        $assign->testable_update_submission($submission, $this->extrastudents[0]->id, true, false);
         // Leave this one as DRAFT.
         $data = new stdClass();
         $data->onlinetext_editor = array('itemid'=>file_get_unused_draft_itemid(),
                                          'text'=>'Submission text',
                                          'format'=>FORMAT_MOODLE);
-        $plugin = $assign1->get_submission_plugin_by_type('onlinetext');
+        $plugin = $assign->get_submission_plugin_by_type('onlinetext');
         $plugin->save($submission, $data);
 
         // Simulate adding a grade.
         $this->setUser($this->teachers[0]);
         $data = new stdClass();
         $data->grade = '50.0';
-        $assign1->testable_apply_grade_to_user($data, $this->extrastudents[0]->id, 0);
+        $assign->testable_apply_grade_to_user($data, $this->extrastudents[0]->id, 0);
 
         // Simulate a submission.
         $this->setUser($this->extrastudents[1]);
-        $submission = $assign1->get_user_submission($this->extrastudents[1]->id, true);
+        $submission = $assign->get_user_submission($this->extrastudents[1]->id, true);
         $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
-        $assign1->testable_update_submission($submission, $this->extrastudents[1]->id, true, false);
+        $assign->testable_update_submission($submission, $this->extrastudents[1]->id, true, false);
         $data = new stdClass();
         $data->onlinetext_editor = array('itemid'=>file_get_unused_draft_itemid(),
                                          'text'=>'Submission text',
                                          'format'=>FORMAT_MOODLE);
-        $plugin = $assign1->get_submission_plugin_by_type('onlinetext');
+        $plugin = $assign->get_submission_plugin_by_type('onlinetext');
         $plugin->save($submission, $data);
 
         // Simulate a submission.
         $this->setUser($this->extrastudents[2]);
-        $submission = $assign1->get_user_submission($this->extrastudents[2]->id, true);
+        $submission = $assign->get_user_submission($this->extrastudents[2]->id, true);
         $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
-        $assign1->testable_update_submission($submission, $this->extrastudents[2]->id, true, false);
+        $assign->testable_update_submission($submission, $this->extrastudents[2]->id, true, false);
         $data = new stdClass();
         $data->onlinetext_editor = array('itemid'=>file_get_unused_draft_itemid(),
                                          'text'=>'Submission text',
                                          'format'=>FORMAT_MOODLE);
-        $plugin = $assign1->get_submission_plugin_by_type('onlinetext');
+        $plugin = $assign->get_submission_plugin_by_type('onlinetext');
         $plugin->save($submission, $data);
 
         // Simulate a submission.
         $this->setUser($this->extrastudents[3]);
-        $submission = $assign1->get_user_submission($this->extrastudents[3]->id, true);
+        $submission = $assign->get_user_submission($this->extrastudents[3]->id, true);
         $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
-        $assign1->testable_update_submission($submission, $this->extrastudents[3]->id, true, false);
+        $assign->testable_update_submission($submission, $this->extrastudents[3]->id, true, false);
         $data = new stdClass();
         $data->onlinetext_editor = array('itemid'=>file_get_unused_draft_itemid(),
                                          'text'=>'Submission text',
                                          'format'=>FORMAT_MOODLE);
-        $plugin = $assign1->get_submission_plugin_by_type('onlinetext');
+        $plugin = $assign->get_submission_plugin_by_type('onlinetext');
         $plugin->save($submission, $data);
 
         // Simulate a submission for suspended user, this will never be counted.
         $this->setUser($this->extrastudents[3]);
-        $submission = $assign1->get_user_submission($this->extrasuspendedstudents[0]->id, true);
+        $submission = $assign->get_user_submission($this->extrasuspendedstudents[0]->id, true);
         $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
-        $assign1->testable_update_submission($submission, $this->extrasuspendedstudents[0]->id, true, false);
+        $assign->testable_update_submission($submission, $this->extrasuspendedstudents[0]->id, true, false);
         $data = new stdClass();
         $data->onlinetext_editor = array('itemid'=>file_get_unused_draft_itemid(),
                                          'text'=>'Submission text',
                                          'format'=>FORMAT_MOODLE);
-        $plugin = $assign1->get_submission_plugin_by_type('onlinetext');
+        $plugin = $assign->get_submission_plugin_by_type('onlinetext');
         $plugin->save($submission, $data);
 
         // Simulate adding a grade.
         $this->setUser($this->teachers[0]);
         $data = new stdClass();
         $data->grade = '50.0';
-        $assign1->testable_apply_grade_to_user($data, $this->extrastudents[3]->id, 0);
-        $assign1->testable_apply_grade_to_user($data, $this->extrasuspendedstudents[0]->id, 0);
+        $assign->testable_apply_grade_to_user($data, $this->extrastudents[3]->id, 0);
+        $assign->testable_apply_grade_to_user($data, $this->extrasuspendedstudents[0]->id, 0);
 
-        $this->assertEquals(2, $assign1->count_grades());
-        $this->assertEquals(4, $assign1->count_submissions());
-        $this->assertEquals(2, $assign1->count_submissions_need_grading());
-        $this->assertEquals(3, $assign1->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_SUBMITTED));
-        $this->assertEquals(1, $assign1->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_DRAFT));
-
-        // Groups.
-        $assign2 = $this->create_instance(array(
-            'assignsubmission_onlinetext_enabled' => 1,
-            'groupmode' => VISIBLEGROUPS
-        ));
-
-        $this->setUser($this->extrastudents[1]);
-        $submission = $assign2->get_user_submission($this->extrastudents[1]->id, true);
-        $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
-        $assign2->testable_update_submission($submission, $this->extrastudents[1]->id, true, false);
-        $data = new stdClass();
-        $data->onlinetext_editor = array('itemid' => file_get_unused_draft_itemid(),
-                                         'text' => 'Submission text',
-                                         'format' => FORMAT_MOODLE);
-        $plugin = $assign2->get_submission_plugin_by_type('onlinetext');
-        $plugin->save($submission, $data);
-
-        $this->assertEquals(1, $assign2->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_SUBMITTED));
-
-        // Set active groups to all groups.
-        $this->setUser($this->teachers[0]);
-        $SESSION->activegroup[$this->course->id]['aag'][0] = 0;
-        $this->assertEquals(1, $assign2->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_SUBMITTED));
-
-        // Set the user group.
-        $studentgroups = groups_get_user_groups($this->course->id, $this->extrastudents[1]->id);
-        $this->assertEquals(1, count($studentgroups));
-        $studentgroup = array_pop($studentgroups);
-        $SESSION->activegroup[$this->course->id]['aag'][0] = $studentgroup[0];
-        $this->assertEquals(1, $assign2->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_SUBMITTED));
-
-        // Set another group.
-        $anothergroup = $this->groups[0];
-        $this->assertNotEquals($anothergroup->id, $studentgroup[0]);
-        $SESSION->activegroup[$this->course->id]['aag'][0] = (int)$anothergroup->id;
-        $this->assertEquals(0, $assign2->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_SUBMITTED));
+        $this->assertEquals(2, $assign->count_grades());
+        $this->assertEquals(4, $assign->count_submissions());
+        $this->assertEquals(2, $assign->count_submissions_need_grading());
+        $this->assertEquals(3, $assign->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_SUBMITTED));
+        $this->assertEquals(1, $assign->count_submissions_with_status(ASSIGN_SUBMISSION_STATUS_DRAFT));
     }
 
     public function test_get_grading_userid_list() {
@@ -842,51 +790,6 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         $this->assertEquals(2, count($messages));
         $this->assertEquals(1, $messages[0]->notification);
         $this->assertEquals($assign->get_instance()->name, $messages[0]->contexturlname);
-
-        // Regrading a grade causes a notification to the user.
-        $data->sendstudentnotifications = true;
-        $assign->testable_apply_grade_to_user($data, $this->students[0]->id, 0);
-        assign::cron();
-        $messages = $sink->get_messages();
-        $this->assertEquals(3, count($messages));
-    }
-
-    /**
-     * Test delivery of grade notifications as controlled by marking workflow.
-     */
-    public function test_markingworkflow_cron() {
-        // First run cron so there are no messages waiting to be sent (from other tests).
-        cron_setup_user();
-        assign::cron();
-
-        // Now create an assignment with marking workflow enabled.
-        $this->setUser($this->editingteachers[0]);
-        $assign = $this->create_instance(array('sendstudentnotifications' => 1, 'markingworkflow' => 1));
-
-        // Simulate adding a grade.
-        $this->setUser($this->teachers[0]);
-        $data = new stdClass();
-        $data->grade = '50.0';
-
-        // This student will not receive notification.
-        $data->workflowstate = ASSIGN_MARKING_WORKFLOW_STATE_READYFORRELEASE;
-        $assign->testable_apply_grade_to_user($data, $this->students[0]->id, 0);
-
-        // This student will receive notification.
-        $data->workflowstate = ASSIGN_MARKING_WORKFLOW_STATE_RELEASED;
-        $assign->testable_apply_grade_to_user($data, $this->students[1]->id, 0);
-
-        // Now run cron and see that one message was sent.
-        $this->preventResetByRollback();
-        $sink = $this->redirectMessages();
-        cron_setup_user();
-        $this->expectOutputRegex('/Done processing 1 assignment submissions/');
-        assign::cron();
-
-        $messages = $sink->get_messages();
-        $this->assertEquals(1, count($messages));
-        $this->assertEquals($messages[0]->useridto, $this->students[1]->id);
-        $this->assertEquals($assign->get_instance()->name, $messages[0]->contexturlname);
     }
 
     public function test_is_graded() {
@@ -1122,12 +1025,11 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         $this->create_extra_users();
         // Now verify group assignments.
         $this->setUser($this->editingteachers[0]);
-        $time = time();
         $assign = $this->create_instance(array('teamsubmission'=>1,
                                                'assignsubmission_onlinetext_enabled'=>1,
                                                'submissiondrafts'=>1,
                                                'requireallteammemberssubmit'=>0,
-                                               'duedate' => $time - 2*24*60*60));
+                                               'duedate' => time() - 2*24*60*60));
         $PAGE->set_url(new moodle_url('/mod/assign/view.php', array('id' => $assign->get_course_module()->id)));
 
         $this->setUser($this->extrastudents[0]);
@@ -1144,8 +1046,7 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         $output = $assign->view_student_summary($this->extrastudents[0], true);
         $this->assertContains(get_string('submitassignment', 'assign'), $output);
         $this->assertContains(get_string('timeremaining', 'assign'), $output);
-        $difftime = time() - $time;
-        $this->assertContains(get_string('overdue', 'assign', format_time(2*24*60*60 + $difftime)), $output);
+        $this->assertContains(get_string('overdue', 'assign', format_time(2*24*60*60)), $output);
 
         $submission = $assign->get_group_submission($this->extrastudents[0]->id, 0, true);
         $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
@@ -1162,8 +1063,7 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
 
         // Check that time remaining is not overdue.
         $this->assertContains(get_string('timeremaining', 'assign'), $output);
-        $difftime = time() - $time;
-        $this->assertContains(get_string('submittedlate', 'assign', format_time(2*24*60*60 + $difftime)), $output);
+        $this->assertContains(get_string('submittedlate', 'assign', format_time(2*24*60*60)), $output);
 
         $submission = $assign->get_group_submission($this->extrastudents[self::GROUP_COUNT]->id, 0, true);
         $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
@@ -1237,18 +1137,72 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         $this->assertCount(10, $assign->testable_get_graders($this->students[1]->id));
     }
 
+    public function test_get_notified_users() {
+        global $CFG, $DB;
+
+        $capability = 'mod/assign:receivegradernotifications';
+        $coursecontext = context_course::instance($this->course->id);
+        $role = $DB->get_record('role', array('shortname' => 'teacher'));
+
+        $this->create_extra_users();
+        $this->setUser($this->editingteachers[0]);
+
+        // Create an assignment with no groups.
+        $assign = $this->create_instance();
+
+        $this->assertCount(self::DEFAULT_TEACHER_COUNT +
+                           self::DEFAULT_EDITING_TEACHER_COUNT +
+                           self::EXTRA_TEACHER_COUNT +
+                           self::EXTRA_EDITING_TEACHER_COUNT,
+                           $assign->testable_get_notifiable_users($this->students[0]->id));
+
+        // Change nonediting teachers role to not receive grader notifications.
+        assign_capability($capability, CAP_PROHIBIT, $role->id, $coursecontext);
+
+        $this->assertCount(self::DEFAULT_EDITING_TEACHER_COUNT +
+                           self::EXTRA_EDITING_TEACHER_COUNT,
+                           $assign->testable_get_notifiable_users($this->students[0]->id));
+
+        // Reset nonediting teachers role to default.
+        unassign_capability($capability, $role->id, $coursecontext);
+
+        // Force create an assignment with SEPARATEGROUPS.
+        $data = new stdClass();
+        $data->courseid = $this->course->id;
+        $data->name = 'Grouping';
+        $groupingid = groups_create_grouping($data);
+        groups_assign_grouping($groupingid, $this->groups[0]->id);
+        $assign = $this->create_instance(array('groupingid' => $groupingid, 'groupmode' => SEPARATEGROUPS));
+
+        $this->setUser($this->students[1]);
+        $this->assertCount(4, $assign->testable_get_notifiable_users($this->students[0]->id));
+        // Note the second student is in a group that is not in the grouping.
+        // This means that we get all graders that are not in a group in the grouping.
+        $this->assertCount(10, $assign->testable_get_notifiable_users($this->students[1]->id));
+
+        // Change nonediting teachers role to not receive grader notifications.
+        assign_capability($capability, CAP_PROHIBIT, $role->id, $coursecontext);
+
+        $this->assertCount(2, $assign->testable_get_notifiable_users($this->students[0]->id));
+        // Note the second student is in a group that is not in the grouping.
+        // This means that we get all graders that are not in a group in the grouping.
+        $this->assertCount(5, $assign->testable_get_notifiable_users($this->students[1]->id));
+    }
+
     public function test_group_members_only() {
         global $CFG;
 
         $this->setAdminUser();
         $this->create_extra_users();
-        $CFG->enablegroupmembersonly = true;
+        $CFG->enableavailability = true;
         $grouping = $this->getDataGenerator()->create_grouping(array('courseid' => $this->course->id));
         groups_assign_grouping($grouping->id, $this->groups[0]->id);
 
         // Force create an assignment with SEPARATEGROUPS.
         $instance = $this->getDataGenerator()->create_module('assign', array('course'=>$this->course->id),
-            array('groupmembersonly' => SEPARATEGROUPS, 'groupingid' => $grouping->id));
+                array('availability' => json_encode(\core_availability\tree::get_root_json(array(
+                    \availability_grouping\condition::get_json()))),
+                'groupingid' => $grouping->id));
 
         $cm = get_coursemodule_from_instance('assign', $instance->id);
         $context = context_module::instance($cm->id);
@@ -1425,180 +1379,25 @@ class mod_assign_locallib_testcase extends mod_assign_base_testcase {
         $formdata->instance = $formdata->id;
         $assign->update_instance($formdata);
 
-        // Check we can repopen still.
+        // Mark the submission again.
+        $data = new stdClass();
+        $data->grade = '60.0';
+        $assign->testable_apply_grade_to_user($data, $this->students[0]->id, 1);
+
+        // Check the grade exists.
+        $grades = $assign->get_user_grades_for_gradebook($this->students[0]->id);
+        $this->assertEquals(60, (int)$grades[$this->students[0]->id]->rawgrade);
+
+        // Check we can reopen still.
         $result = $assign->testable_process_add_attempt($this->students[0]->id);
         $this->assertEquals(true, $result);
 
+        // Should no longer have a grade because there is no grade for the latest attempt.
         $grades = $assign->get_user_grades_for_gradebook($this->students[0]->id);
-        $this->assertEquals(50, (int)$grades[$this->students[0]->id]->rawgrade);
+        $this->assertEmpty($grades);
 
     }
 
-    /**
-     * Test reopen behavior when in "Reopen until pass" mode.
-     */
-    public function test_attempt_reopen_method_untilpass() {
-        global $PAGE;
-
-        $this->setUser($this->editingteachers[0]);
-        $assign = $this->create_instance(array('attemptreopenmethod' => ASSIGN_ATTEMPT_REOPEN_METHOD_UNTILPASS,
-                'maxattempts' => 3,
-                'submissiondrafts' => 1,
-                'assignsubmission_onlinetext_enabled' => 1));
-        $PAGE->set_url(new moodle_url('/mod/assign/view.php', array('id' => $assign->get_course_module()->id)));
-
-        // Set grade to pass to 80.
-        $gradeitem = $assign->get_grade_item();
-        $gradeitem->gradepass = '80.0';
-        $gradeitem->update();
-
-        // Student should be able to see an add submission button.
-        $this->setUser($this->students[0]);
-        $output = $assign->view_student_summary($this->students[0], true);
-        $this->assertNotEquals(false, strpos($output, get_string('addsubmission', 'assign')));
-
-        // Add a submission.
-        $now = time();
-        $submission = $assign->get_user_submission($this->students[0]->id, true);
-        $data = new stdClass();
-        $data->onlinetext_editor = array('itemid' => file_get_unused_draft_itemid(),
-                'text' => 'Submission text',
-                'format' => FORMAT_MOODLE);
-        $plugin = $assign->get_submission_plugin_by_type('onlinetext');
-        $plugin->save($submission, $data);
-
-        // And now submit it for marking.
-        $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
-        $assign->testable_update_submission($submission, $this->students[0]->id, true, false);
-
-        // Verify the student cannot make a new attempt.
-        $output = $assign->view_student_summary($this->students[0], true);
-        $this->assertEquals(false, strpos($output, get_string('addnewattempt', 'assign')));
-
-        // Mark the submission as non-passing.
-        $this->setUser($this->teachers[0]);
-        $data = new stdClass();
-        $data->grade = '50.0';
-        $assign->testable_apply_grade_to_user($data, $this->students[0]->id, 0);
-
-        // Check the student can see the grade.
-        $this->setUser($this->students[0]);
-        $output = $assign->view_student_summary($this->students[0], true);
-        $this->assertNotEquals(false, strpos($output, '50.0'));
-
-        // Check that the student now has a button for Add a new attempt.
-        $output = $assign->view_student_summary($this->students[0], true);
-        $this->assertNotEquals(false, strpos($output, get_string('addnewattempt', 'assign')));
-
-        // Check that the student now does not have a button for Submit.
-        $this->assertEquals(false, strpos($output, get_string('submitassignment', 'assign')));
-
-        // Check that the student now has a submission history.
-        $this->assertNotEquals(false, strpos($output, get_string('attempthistory', 'assign')));
-
-        // Add a second submission.
-        $now = time();
-        $submission = $assign->get_user_submission($this->students[0]->id, true, 1);
-        $data = new stdClass();
-        $data->onlinetext_editor = array('itemid' => file_get_unused_draft_itemid(),
-                'text' => 'Submission text',
-                'format' => FORMAT_MOODLE);
-        $plugin = $assign->get_submission_plugin_by_type('onlinetext');
-        $plugin->save($submission, $data);
-
-        // And now submit it for marking.
-        $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
-        $assign->testable_update_submission($submission, $this->students[0]->id, true, false);
-
-        // Mark the submission as passing.
-        $this->setUser($this->teachers[0]);
-        $data = new stdClass();
-        $data->grade = '80.0';
-        $assign->testable_apply_grade_to_user($data, $this->students[0]->id, 1);
-
-        // Check that the student does not have a button for Add a new attempt.
-        $this->setUser($this->students[0]);
-        $output = $assign->view_student_summary($this->students[0], true);
-        $this->assertEquals(false, strpos($output, get_string('addnewattempt', 'assign')));
-
-        // Re-mark the submission as not passing.
-        $this->setUser($this->teachers[0]);
-        $data = new stdClass();
-        $data->grade = '50.0';
-        $assign->testable_apply_grade_to_user($data, $this->students[0]->id, 1);
-
-        // Check that the student now has a button for Add a new attempt.
-        $this->setUser($this->students[0]);
-        $output = $assign->view_student_summary($this->students[0], true);
-        $this->assertNotEquals(false, strpos($output, get_string('addnewattempt', 'assign')));
-
-        // Add a submission as a second student.
-        $this->setUser($this->students[1]);
-        $now = time();
-        $submission = $assign->get_user_submission($this->students[1]->id, true);
-        $data = new stdClass();
-        $data->onlinetext_editor = array('itemid' => file_get_unused_draft_itemid(),
-                'text' => 'Submission text',
-                'format' => FORMAT_MOODLE);
-        $plugin = $assign->get_submission_plugin_by_type('onlinetext');
-        $plugin->save($submission, $data);
-
-        // And now submit it for marking.
-        $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
-        $assign->testable_update_submission($submission, $this->students[1]->id, true, false);
-
-        // Mark the submission as passing.
-        $this->setUser($this->teachers[0]);
-        $data = new stdClass();
-        $data->grade = '100.0';
-        $assign->testable_apply_grade_to_user($data, $this->students[1]->id, 0);
-
-        // Check the student can see the grade.
-        $this->setUser($this->students[1]);
-        $output = $assign->view_student_summary($this->students[1], true);
-        $this->assertNotEquals(false, strpos($output, '100.0'));
-
-        // Check that the student does not have a button for Add a new attempt.
-        $output = $assign->view_student_summary($this->students[1], true);
-        $this->assertEquals(false, strpos($output, get_string('addnewattempt', 'assign')));
-
-        // Set grade to pass to 0, so that no attempts should reopen.
-        $gradeitem = $assign->get_grade_item();
-        $gradeitem->gradepass = '0';
-        $gradeitem->update();
-
-        // Add another submission.
-        $this->setUser($this->students[2]);
-        $now = time();
-        $submission = $assign->get_user_submission($this->students[2]->id, true);
-        $data = new stdClass();
-        $data->onlinetext_editor = array('itemid' => file_get_unused_draft_itemid(),
-                'text' => 'Submission text',
-                'format' => FORMAT_MOODLE);
-        $plugin = $assign->get_submission_plugin_by_type('onlinetext');
-        $plugin->save($submission, $data);
-
-        // And now submit it for marking.
-        $submission->status = ASSIGN_SUBMISSION_STATUS_SUBMITTED;
-        $assign->testable_update_submission($submission, $this->students[2]->id, true, false);
-
-        // Mark the submission as graded.
-        $this->setUser($this->teachers[0]);
-        $data = new stdClass();
-        $data->grade = '0.0';
-        $assign->testable_apply_grade_to_user($data, $this->students[2]->id, 0);
-
-        // Check the student can see the grade.
-        $this->setUser($this->students[2]);
-        $output = $assign->view_student_summary($this->students[2], true);
-        $this->assertNotEquals(false, strpos($output, '0.0'));
-
-        // Check that the student does not have a button for Add a new attempt.
-        $output = $assign->view_student_summary($this->students[2], true);
-        $this->assertEquals(false, strpos($output, get_string('addnewattempt', 'assign')));
-    }
-
-
     public function test_markingworkflow() {
         global $PAGE;
 
@@ -2032,5 +1831,46 @@ Anchor link 2:<a title=\"bananas\" href=\"../logo-240x60.gif\">Link text</a>
         $this->assertTrue($assign->can_edit_submission($this->students[0]->id, $this->extrastudents[0]->id));
 
     }
+
+    /**
+     * Test if the view blind details capability works
+     */
+    public function test_can_view_blind_details() {
+        global $PAGE, $DB;
+        $teacherrole = $DB->get_record('role', array('shortname' => 'teacher'));
+        $managerrole = $DB->get_record('role', array('shortname' => 'manager'));
+
+        $student = $this->students[0];// Get a student user.
+        // Create a teacher. Shouldn't be able to view blind marking ID.
+        $teacher = $this->getDataGenerator()->create_user();
+
+        $this->getDataGenerator()->enrol_user($teacher->id,
+                                              $this->course->id,
+                                              $teacherrole->id);
+
+        // Create a manager.. Should be able to view blind marking ID.
+        $manager = $this->getDataGenerator()->create_user();
+        $this->getDataGenerator()->enrol_user($manager->id,
+                $this->course->id,
+                $managerrole->id);
+
+        // Generate blind marking assignment.
+        $assign = $this->create_instance(array('blindmarking' => 1));
+        $this->assertEquals(true, $assign->is_blind_marking());
+
+        // Test student names are hidden to teacher.
+        $this->setUser($teacher);
+        $gradingtable = new assign_grading_table($assign, 1, '', 0, true);
+        $output = $assign->get_renderer()->render($gradingtable);
+        $this->assertEquals(true, strpos($output, get_string('hiddenuser', 'assign')));    // "Participant" is somewhere on the page.
+        $this->assertEquals(false, strpos($output, fullname($student)));    // Students full name doesn't appear.
+
+        // Test student names are visible to manager.
+        $this->setUser($manager);
+        $gradingtable = new assign_grading_table($assign, 1, '', 0, true);
+        $output = $assign->get_renderer()->render($gradingtable);
+        $this->assertEquals(false, strpos($output, get_string('hiddenuser', 'assign')));
+        $this->assertEquals(true, strpos($output, fullname($student)));    //students full name doesn't appear.
+    }
 }
 
